import wslite.soap.SOAPClient
/**
 * @author Varban Vassilev (vvv@urbancode.com)
 */

public class NetScalerHelper {

    def private SOAPClient client
    def private cookie

    def public NetScalerHelper(serverUrl) {
        // fix URL formatting and remove trailing slashes
        serverUrl = serverUrl?.replaceAll('\\\\', '/')
        while (serverUrl?.endsWith('/')) {
            serverUrl = serverUrl[0..-2]
        }
        serverUrl += '/soap'

        client = new SOAPClient(serverUrl)
        client?.httpClient?.sslTrustAllCerts = true
    }

    def public login = { user, pass ->
        println "Login to server"
        def response = sendMessage {
            body {
                'login' {
                    username(user)
                    password(pass)
                }
            }
        }
        def returnCode = response.loginResponse.'return'.rc.text()
        def message = response.loginResponse.'return'.message.text()
        cookie = response.httpResponse.headers['Set-Cookie']
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error logging in: $returnCode - $message")
        }
    }

    def public logout = { ->
        println "Logout from server"
        def response = sendMessage {
            body {
                'logout' {}
            }
        }
        def returnCode = response.logoutResponse.'return'.rc.text()
        def message = response.logoutResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error logging out: $returnCode - $message")
        }
    }

    def public enableService = { service ->
        println "Enable service $service"
        def response = sendMessage {
            body {
                'enableservice' {
                    name(service)
                }
            }
        }
        def returnCode = response.enableserviceResponse.'return'.rc.text()
        def message = response.enableserviceResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error enabling service: $returnCode - $message")
        }
    }

    def public disableService = { service, timeou ->
        println "Disable service $service with timeout $timeout (s)"
        def response = sendMessage {
            body {
                'disableservice' {
                    name(service)
                    delay(timeout)
                }
            }
        }
        def returnCode = response.disableserviceResponse.'return'.rc.text()
        def message = response.disableserviceResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error disabling service: $returnCode - $message")
        }
    }

        def public enableServiceGroup = { serviceGroup ->
        println "Enable service group $serviceGroup"
        def response = sendMessage {
            body {
                'enableservicegroup' {
                    servicegroupname(serviceGroup)
                }
            }
        }
        def returnCode = response.enableservicegroupResponse.'return'.rc.text()
        def message = response.enableservicegroupResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error enabling service group: $returnCode - $message")
        }
    }

    def public disableServiceGroup = { serviceGroup, timeout ->
        println "Disable service group $serviceGroup with timeout $timeout (s)"
        def response = sendMessage {
            body {
                'disableservicegroup' {
                    servicegroupname(serviceGroup)
                    delay(timeout)
                }
            }
        }
        def returnCode = response.disableservicegroupResponse.'return'.rc.text()
        def message = response.disableservicegroupResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error disabling service group: $returnCode - $message")
        }
    }

    def public enableServer = { server ->
        println "Enable server $server"
        def response = sendMessage {
            body {
                'enableserver' {
                    name(server)
                }
            }
        }
        def returnCode = response.enableserverResponse.'return'.rc.text()
        def message = response.enableserverResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error enabling server: $returnCode - $message")
        }
    }

    def public disableServer = { server, timeout ->
        println "Disable server $server with timeout $timeout (s)"
        def response = sendMessage {
            body {
                'disableserver' {
                    name(server)
                    delay(timeout)
                }
            }
        }
        def returnCode = response.disableserverResponse.'return'.rc.text()
        def message = response.disableserverResponse.'return'.message.text()
        if (returnCode.toInteger() != 0) {
            throw new Exception("Error disabling server: $returnCode - $message")
        }
    }

    def private sendMessage = { Closure closure ->
        if (cookie) {
            client.send(headers:['Set-Cookie':cookie], closure)
        }
        else {
            client.send(closure)
        }
    }
}
